@@ -2,14 +2,16 @@ package Net::FTP::Common;
 
 use strict;
 
+use Carp qw(cluck confess);
 use Data::Dumper;
 use Net::FTP;
 
+
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::FTP);
 
-$VERSION = sprintf '%s', q{$Revision: 2.9 $} =~ /\S+\s+(\S+)/ ;
+$VERSION = '3.0';
 
 # Preloaded methods go here.
 
@@ -22,12 +24,13 @@ sub new {
     (
      Host => 'ftp.microsoft.com',
      RemoteDir  => '/pub',
+     LocalDir  => '.',   # setup something for $ez->get
      Type => 'I'
     );
 
-  my %netftp_cfg_default = ( Debug => 1, Timeout => 240 );
+  my %netftp_cfg_default = ( Debug => 1, Timeout => 240, Passive => 1 );
 
-  # overwrite defaults with values supplied by sub input
+  # overwrite defaults with values supplied by constructor input
   @common_cfg_default{keys %$common_cfg_in} = values %$common_cfg_in;
   @netftp_cfg_default{keys  %netftp_cfg_in} = values  %netftp_cfg_in;
     
@@ -38,13 +41,33 @@ sub new {
 
   my $new_self = { %$self, Common => $self->{Common} } ;
 
+  if (my $file = $self->{Common}{STDERR}) {
+      open DUP, ">$file" or die "cannot dup STDERR to $file: $!";
+      lstat DUP; # kill used only once error
+      open STDERR, ">&DUP";
+  }
+
+
   bless $new_self, $pkg;
 }
 
 sub Common {
     my $self = shift;
+
+    not (@_ % 2) or die 
+"
+Odd number of elements in assignment hash in call to Common().
+Common() is a 'setter' subroutine. You cannot call it with an
+odd number of arguments (e.g. $self->Common('Type') ) and 
+expect it to get a value. use GetCommon() for that.
+
+Here is what you passed in.
+", Dumper(\@_);
+
     my %tmp = @_;
 
+#    warn "HA: ", Dumper(\%tmp,\@_);
+
     @{$self->{Common}}{keys %tmp} = values %tmp;
 }
 
@@ -52,7 +75,11 @@ sub GetCommon {
     my ($self,$key) = @_;
 
     if ($key) {
-	$self->{Common}{$key};
+	if (defined($self->{Common}{$key})) {
+	    return ($self->{Common}{$key});
+	} else {
+	    return undef;
+	}
     } else {
 	$self->{Common};
     }
@@ -62,9 +89,9 @@ sub Host { $_[0]->{Common}->{Host} or die "Host must be defined when creating a
 
 sub NetFTP { 
 
-my ($self, %config) = @_;
+    my ($self, %config) = @_;
 
-@{$self}{keys %config} = values %config;
+    @{$self}{keys %config} = values %config;
 
 }
 
@@ -74,20 +101,26 @@ sub login {
 #  my $ftp_session = Net::FTP->new($self->Host, %{$self->{NetFTP}});
   my $ftp_session = Net::FTP->new($self->Host, %$self);
 
-  $ftp_session or return undef;
+#  $ftp_session or return undef;
+  $ftp_session or 
+      die 'FATAL: attempt to create Net::FTP session failed.
+Most likely reason for this is lack of internet connectivity.
+';
 
   my $session;
+  my $account = $self->GetCommon('Account');
   if ($self->GetCommon('User') and $self->GetCommon('Pass')) {
       $session = 
 	  $ftp_session->login($self->GetCommon('User') , 
-			      $self->GetCommon('Pass'));
+			      $self->GetCommon('Pass'),
+			      $account);
   } else {
       warn "either User or Pass was not defined. Attempting .netrc for login";
       $session = 
 	  $ftp_session->login;
   }
 
-  $session and return $ftp_session 
+  $session and ($self->Common('FTPSession', $ftp_session)) and return $ftp_session 
       or 
 	  warn "error logging in: $!" and return undef;
 
@@ -97,7 +130,6 @@ sub ls {
   my ($self, @config) = @_;
   my %config=@config;
 
-
   my $ftp = $self->prep(%config);
 
   my $ls = $ftp->ls;
@@ -125,7 +157,8 @@ sub dir {
     my %HoH;
     foreach (@{$dir})
         {
-        $_ =~ m#([a-z-]*)\s*([0-9]*)\s*([0-9a-zA-Z]*)\s*([0-9a-zA-Z]*)\s*([0-9]*)\s*([A-Za-z]*)\s*([0-9]*)\s*([0-9A-Za-z:]*)\s*([A-Za-z0-9.-]*)#;
+	    # $_ =~ m#([a-z-]*)\s*([0-9]*)\s*([0-9a-zA-Z]*)\s*([0-9a-zA-Z]*)\s*([0-9]*)\s*([A-Za-z]*)\s*([0-9]*)\s*([0-9A-Za-z:]*)\s*([A-Za-z0-9.-]*)#;
+	    $_ = m#([a-z-]*)\s*([0-9]*)\s*([0-9a-zA-Z]*)\s*([0-9a-zA-Z]*)\s*([0-9]*)\s*([A-Za-z]*)\s*([0-9]*)\s*([0-9A-Za-z:]*)\s*([\w*\W*\s*\S*]*)#;
 
         my $perm = $1;
         my $inode = $2;
@@ -162,8 +195,9 @@ sub mkdir {
     my ($self,%config) = @_;
 
     my $ftp = $self->prep(%config);
-
-    $ftp->mkdir($self->GetCommon('RemoteDir'), $self->GetCommon('Recurse'));
+    my $rd =  $self->GetCommon('RemoteDir');
+    my $r  =  $self->GetCommon('Recurse');
+    $ftp->mkdir($rd, $r);
 }
 
 # The Perl -e operator for files on a remote site. Even though the
@@ -175,7 +209,11 @@ sub exists {
 
     my @listing = $self->ls(%cfg);
 
-    scalar grep { $_ eq $self->GetCommon('File') } @listing;
+    my $rf = $self->GetCommon('RemoteFile');
+
+    warn sprintf "[checking @listing for [%s]]", $rf;
+
+    scalar grep { $_ eq $self->GetCommon('RemoteFile') } @listing;
 }
 
 sub grep {
@@ -189,19 +227,40 @@ sub grep {
     grep { $_ =~ /$cfg{Grep}/ } @listing;
 }
 
+sub connected {
+    my $self = shift;
 
-# Really, the best abstraction would be:
-# ftp_download_all_with_hook where the hook in this case would be a
-# coderef that conditionally decrypts the downloaded file
-# Laziness now will lead to more work later I guess.
+#    warn "CONNECTED SELF ", Dumper($self);
+
+    my $session = $self->GetCommon('FTPSession') or return 0;
+
+    local $@;
+    my $pwd;
+    my $connected = $session->pwd ? 1 : 0;
+#    warn "connected: $connected RESP: $connected";
+    $connected;
+}
+
+sub prepped {
+    my $self = shift; 
+    my $prepped = $self->GetCommon('FTPSession') and $self->connected;
+#    warn "prepped: $prepped";
+    $prepped;
+}
 
 sub prep {
   my $self = shift;
   my %cfg  = @_;
 
   $self->Common(%cfg);
-  
-  my $ftp = $self->login;
+
+# This will not work if the Host changes and you are still connected to the prior host.
+# It might be wise to simply drop connection if the Host key changes, but I don't think
+# I will go there right now.
+#  my $ftp = $self->connected ? $self->GetCommon('FTPSession') : $self->login ;
+# So instead:
+  my $ftp = $self->login ;
+
   if ($self->{Common}->{RemoteDir}) {
       $ftp->cwd($self->GetCommon('RemoteDir'))
   } else {
@@ -232,20 +291,42 @@ sub get {
 
   my $r;
 
-  my $file = $self->GetCommon('LocalFile') 
-      ? $self->GetCommon('LocalFile') : $self->GetCommon('File') ;
+  my $file;
+  if ($self->GetCommon('LocalFile')) {
+      $file= $self->GetCommon('LocalFile');
+  } else {
+      $file=$self->GetCommon('RemoteFile');
+  }
+	
   my $local_file = join '/', ($self->GetCommon('LocalDir'), $file);
 		
+#  warn "LF: $local_file ", "D: ", Dumper($self);
 
-  if ($r = $ftp->get($self->GetCommon('File'), $local_file)) {
+
+  if ($r = $ftp->get($self->GetCommon('RemoteFile'), $local_file)) {
       return $r;
   } else { 
-    warn sprintf "download of %s to %s failed",
-	  $self->GetCommon('File'), $self->GetCommon('LocalFile');
+    warn sprintf 'download of %s to %s failed',
+	  $self->GetCommon('RemoteFile'), $self->GetCommon('LocalFile');
+    warn 
+	'here are the settings in your Net::FTP::Common object: %s',
+	Dumper($self);
     return undef;
 }
+  
 
+}
 
+sub file_attr {
+    my $self = shift;
+    my %hash;
+    my @key = qw(LocalFile LocalDir RemoteFile RemoteDir);
+    @hash{@key} = @{$self->{Common}}{@key};
+    %hash;
+}
+
+sub bad_filename {
+    shift =~ /[\r\n]/s;
 }
 
 sub send {
@@ -253,8 +334,32 @@ sub send {
 
   my $ftp = $self->prep(%cfg);
 
-  $ftp->put($self->GetCommon('File')) or
-      die sprintf "upload of %s failed", $self->GetCommon('File');
+#  warn "send_self", Dumper($self);
+
+  my %fa = $self->file_attr;
+
+  if (bad_filename($fa{LocalFile})) {
+      warn "filenames may not have CRLF in them. skipping $fa{LocalFile}";
+      return;
+  }
+
+#  warn "send_fa: ", Dumper(\%fa);
+
+  my $lf = sprintf "%s/%s", $fa{LocalDir}, $fa{LocalFile};
+  my $RF = $fa{RemoteFile} ? $fa{RemoteFile} : $fa{LocalFile};
+  my $rf = sprintf "%s/%s", $fa{RemoteDir}, $RF;
+
+#  warn "[upload $lf as $rf]";
+
+  $ftp->put($lf, $RF) or 
+      confess sprintf "upload of %s to %s failed", $lf, $rf;
+}
+
+sub put { goto &send }
+
+sub DESTROY {
+
+
 }
 
 
@@ -263,7 +368,7 @@ __END__
 
 =head1 NAME
 
-Net::FTP::Common - Perl extension for simplifying common usages of Net::FTP.
+Net::FTP::Common - simplify common usages of Net::FTP
 
 =head1 SYNOPSIS
 
@@ -283,7 +388,7 @@ Net::FTP::Common - Perl extension for simplifying common usages of Net::FTP.
      # Other options
      #
 
-     LocalDir  => '/tmp/downloads'   # setup something for $ez->get
+
      LocalFile => 'delete.zip'       # setup something for $ez->get
      Host => 'ftp.fcc.gov',          # overwrite ftp.microsoft.com default
      RemoteDir  => '/',                    # automatic CD on remote machine to RemoteDir
@@ -320,7 +425,7 @@ Net::FTP::Common - Perl extension for simplifying common usages of Net::FTP.
 
   # Let's list the default dir on several hosts
  
- @host_listings = map { $ez->ls(Host => $_) } @host_list
+  @host_listings = map { $ez->ls(Host => $_) } @host_list
 
   # Let's get the listings of several directories
 
@@ -357,22 +462,22 @@ Net::FTP::Common - Perl extension for simplifying common usages of Net::FTP.
 
   # Get a file from the remote machine
 
-  $ez->get(File => 'codex.txt', LocalFile => '/tmp/crypto.txt');
+  $ez->get(RemoteFile => 'codex.txt', LocalFile => '/tmp/crypto.txt');
 
   # Get a file from the remote machine, specifying dir:
-  $ez->get(File => 'codex.txt', LocalDir => '/tmp');
+  $ez->get(RemoteFile => 'codex.txt', LocalDir => '/tmp');
 
   # NOTE WELL:  because the prior call set LocalFile, it is still a
   # part of the object store. In other words this example will try
   # to store the downloaded file in /tmp/tmp/crypto.txt.
   # Better to say:
 
-  $ez->get(File => 'codex.txt', LocalDir => '/tmp', LocalFile => '');
+  $ez->get(RemoteFile => 'codex.txt', LocalDir => '/tmp', LocalFile => '');
 
 
   # Send a file to the remote machine (*dont* use put!)
 
-  $ez->send(File => 'codex.txt');
+  $ez->send(RemoteFile => 'codex.txt');
 
   # test for a file's existence on the remote machine (using =~)
 
@@ -381,11 +486,15 @@ Net::FTP::Common - Perl extension for simplifying common usages of Net::FTP.
 
   # test for a file on the remote machine (using eq)
 
-  $ez->exists(File => 'needed-file.txt');
+  $ez->exists(RemoteFile => 'needed-file.txt');
 
   # note this is no more than you manually calling:
   # (scalar grep { $_ eq 'needed-file.txt' } $ez->ls) > 0;
 
+  # Let's get all output written to STDERR to goto a logfile
+
+  my $ez = Net::FTP::Common->new( { %CFG, STDERR => $logfile }, %netftp_cfg);
+
 The test suite contains plenty of common examples.
 
 =head1 DESCRIPTION
@@ -419,14 +528,6 @@ Note well that you NEVER have to login first. All API functions automatically lo
 specified directory. However, sometimes it is useful to see if you can actually login before
 attempting to do something else on an FTP site. This is the only time you will need the login() API method.
 
-Note well: though Net::FTP works in the stateful way that the FTP protocol 
-does, Net::FTP::Common works in a stateless "one-hit" fashion. That is, for
-each separate call to the API, a connection is established, the particular
-Net::FTP::Common functionality is performed and the connection is dropped.
-The disadvantage of this approach is the (usually irrelevant and 
-insignificant) overhead of connection and disconnection. The
-advantage is that there is much less chance of incurring failure due
-to timeout.
 
 =head1 METHODS
 
@@ -517,8 +618,8 @@ You may give this function any number of configuration arguments to over-ride th
 Here is the results of the example from the the test suite (t/dir.t):
 
  my %retval = $ez->dir;
- use Data::Dumper;
- warn "NEW_DIR ...", Dumper(\%retval);
+
+# warn "NEW_DIR ...", Dumper(\%retval);
 
           'incoming' => {
                           'owner' => 'root',
@@ -605,12 +706,12 @@ Here is the results of the example from the the test suite (t/dir.t):
 Makes directories on remote FTP server. Will recurse if Recurse => 1 is
 in object's internal state of overridden at method call time. 
 
-Note this function has no test case. Good luck with it. Report any 
-difficulties please.
+This function has no test case but a working example of its use is in 
+C<scripts/rsync.pl>. I use it to back up my stuff.
 
 =head2 $ez->exists (%override)
 
-uses the C<File> option of object internal state (or override) to check for a
+uses the C<RemoteFile> option of object internal state (or override) to check for a
 file in a directory listing. This means a C<eq>, not regex match.
 
 =head2 $ez->grep(%override)
@@ -623,7 +724,7 @@ file in a directory listing. This means a regex, not C<eq> match.
 =head2 $ez->get(%override)
 
 
-uses the C<File>, C<LocalFile>, and C<LocalDir> options of object internal 
+uses the C<RemoteFile>, C<LocalFile>, and C<LocalDir> options of object internal 
 state 
 (or override) to download a file. No slashes need be appended to the end of
 C<LocalDir>. If C<LocalFile> and C<LocalDir> arent defined, then the file
@@ -660,25 +761,62 @@ supposed to.
 
 =over 4
 
+=item * A good example of Net::FTP::Common usage comes with your download:
+
+C<scripts/rsync.pl>
+
+Although this script
+requires AppConfig, Net::FTP::Common in general does not... but go get
+AppConfig anyway, it rocks the house.
+
 =item * A slide talk on Net::FTP::Common in HTML format is available at
 
   http://www.metaperl.com
 
-=item * big change from version after 2.30:
+=item * subscribe to the mailing list via
 
-C<Dir> has been changed to C<RemoteDir> to avoid confusion.
-
-=item * on object destruction:
-
-When a Net::FTP::Common object is goes out of scope, the following
-warning is thrown by Net::FTP:
-
- (in cleanup) Not a GLOB reference at Net/FTP.pm line 147.
-
-This is a harmless error that I should fix some day.
+net-ftp-common-subscribe@yahoogroups.com
 
 =back
 
+=head1 TODO
+
+=head2 Definite
+
+
+=over 4
+
+=item * support resumeable downloads
+
+=back
+
+=head2 Musings
+
+=over 4
+
+=item * Cache directory listings?
+
+=item * parsing FTP list output
+
+This output is not standard. We did a fair job for most common Unixen, but
+if we aspire to the heights of an ange-ftp or other high-quality FTP
+client, we need something like they have in python:
+
+     http://freshmeat.net/redir/ftpparsemodule/20709/url_homepage/
+
+
+=back
+
+=head1 OTHER FTP OFFERINGS
+
+=over 4
+
+=item * http://lftp.yar.ru
+
+
+=back
+
+
 =head1 AUTHOR
 
 T. M. Brannon <tbone@cpan.org>
